# Copyright 2025 Ant Group Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Dialect State Protocol definitions.

Dialects can be stateful or stateless. Stateful dialects maintain
runtime state that is attached to an Interpreter instance.

Usage:
    # Attach state to interpreter
    interpreter.set_dialect_state("simp", simp_state)

    # Retrieve in kernel
    state = interpreter.get_dialect_state("simp")
"""

from __future__ import annotations

from typing import Protocol, runtime_checkable


@runtime_checkable
class DialectState(Protocol):
    """Base protocol for dialect states.

    Dialects that maintain runtime state should implement this protocol.
    Stateless dialects (like tensor, field) don't need a state object.
    """

    dialect_name: str
